@@ -44,6 +44,7 @@ extern int cfg_key_aggregate(char *, char *, char *);
 extern int cfg_key_aggregate_primitives(char *, char *, char *);
 extern int cfg_key_snaplen(char *, char *, char *);
 extern int cfg_key_aggregate_filter(char *, char *, char *);
+extern int cfg_key_dtls_path(char *, char *, char *);
 extern int cfg_key_pcap_filter(char *, char *, char *);
 extern int cfg_key_pcap_protocol(char *, char *, char *);
 extern int cfg_key_pcap_savefile(char *, char *, char *);
@@ -168,6 +169,7 @@ extern int cfg_key_nfacctd_kafka_broker_port(char *, char *, char *);
 extern int cfg_key_nfacctd_kafka_topic(char *, char *, char *);
 extern int cfg_key_nfacctd_kafka_config_file(char *, char *, char *);
 extern int cfg_key_nfacctd_zmq_address(char *, char *, char *);
+extern int cfg_key_nfacctd_dtls_port(char *, char *, char *);
 extern int cfg_key_pmacctd_force_frag_handling(char *, char *, char *);
 extern int cfg_key_pmacctd_frag_buffer_size(char *, char *, char *);
 extern int cfg_key_pmacctd_flow_buffer_size(char *, char *, char *);
@@ -285,6 +287,8 @@ extern int cfg_key_nfprobe_timeouts(char *, char *, char *);
 extern int cfg_key_nfprobe_hoplimit(char *, char *, char *);
 extern int cfg_key_nfprobe_maxflows(char *, char *, char *);
 extern int cfg_key_nfprobe_receiver(char *, char *, char *);
+extern int cfg_key_nfprobe_dtls(char *, char *, char *);
+extern int cfg_key_nfprobe_dtls_verify_cert(char *, char *, char *);
 extern int cfg_key_nfprobe_version(char *, char *, char *);
 extern int cfg_key_nfprobe_engine(char *, char *, char *);
 extern int cfg_key_nfprobe_peer_as(char *, char *, char *);
@@ -462,6 +466,7 @@ extern int cfg_key_bmp_daemon_dump_kafka_partition(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_kafka_partition_key(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_kafka_config_file(char *, char *, char *);
 extern int cfg_key_bmp_daemon_dump_kafka_avro_schema_registry(char *, char *, char *);
+extern int cfg_key_nfacctd_bmp_daemon_parse_proxy_header(char *, char *, char *);
 extern int cfg_key_nfacctd_flow_to_rd_map(char *, char *, char *);
 extern int cfg_key_nfacctd_isis(char *, char *, char *);
 extern int cfg_key_nfacctd_isis_ip(char *, char *, char *);
