@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -19,14 +19,24 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#define __IMT_PLUGIN_C
-
 /* includes */
 #include "pmacct.h"
 #include "plugin_hooks.h"
 #include "plugin_common.h"
 #include "imt_plugin.h"
 #include "bgp/bgp.h"
+#include "net_aggr.h"
+#include "ports_aggr.h"
+
+//Global variables
+void (*imt_insert_func)(struct primitives_ptrs *);
+unsigned char *mpd;
+unsigned char *a;
+struct memory_pool_desc *current_pool;
+struct acc **lru_elem_ptr;
+int no_more_space;
+struct timeval cycle_stamp;
+struct timeval table_reset_stamp;
 
 /* Functions */
 void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr) 
@@ -50,17 +60,17 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   int pollagain = 0;
   u_int32_t seq = 0;
   int rg_err_count = 0;
-  int ret, lock = FALSE, cLen, num, sd, sd2;
+  int ret, lock = FALSE, num, sd, sd2;
   struct pkt_bgp_primitives *pbgp, empty_pbgp;
   struct pkt_legacy_bgp_primitives *plbgp, empty_plbgp;
   struct pkt_nat_primitives *pnat, empty_pnat;
   struct pkt_mpls_primitives *pmpls, empty_pmpls;
   struct pkt_tunnel_primitives *ptun, empty_ptun;
-  char *pcust, empty_pcust[] = "";
+  unsigned char *pcust, empty_pcust[] = "";
   struct pkt_vlen_hdr_primitives *pvlen, empty_pvlen;
   struct networks_file_data nfd;
   struct primitives_ptrs prim_ptrs;
-  struct plugins_list_entry *plugin_data = ((struct channels_list_entry *)ptr)->plugin;
+  socklen_t cLen;
 
   /* poll() stuff */
   struct pollfd poll_fd[2]; /* pipe + server */
@@ -72,14 +82,16 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   void *zmq_host = NULL;
 #endif
 
+#ifdef WITH_REDIS
+  struct p_redis_host redis_host;
+#endif
+
   memcpy(&config, cfgptr, sizeof(struct configuration));
   memcpy(&extras, &((struct channels_list_entry *)ptr)->extras, sizeof(struct extra_primitives));
   recollect_pipe_memory(ptr);
   pm_setproctitle("%s [%s]", "IMT Plugin", config.name);
 
   if (config.proc_priority) {
-    int ret;
-
     ret = setpriority(PRIO_PROCESS, 0, config.proc_priority);
     if (ret) Log(LOG_WARNING, "WARN ( %s/%s ): proc_priority failed (errno: %d)\n", config.name, config.type, errno);
     else Log(LOG_INFO, "INFO ( %s/%s ): proc_priority set to %d\n", config.name, config.type, getpriority(PRIO_PROCESS, 0));
@@ -134,7 +146,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
   if (!config.memory_pool_size) config.memory_pool_size = MEMORY_POOL_SIZE;  
   else {
     if (config.memory_pool_size < sizeof(struct acc)) {
-      Log(LOG_WARNING, "WARN ( %s/%s ): enforcing memory pool's minimum size, %d bytes.\n", config.name, config.type, sizeof(struct acc));
+      Log(LOG_WARNING, "WARN ( %s/%s ): enforcing memory pool's minimum size, %d bytes.\n", config.name, config.type, (int)sizeof(struct acc));
       config.memory_pool_size = MEMORY_POOL_SIZE;
     }
   }
@@ -190,6 +202,15 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
   qh = (struct query_header *) srvbuf;
 
+#ifdef WITH_REDIS
+  if (config.redis_host) {
+    char log_id[SHORTBUFLEN];
+
+    snprintf(log_id, sizeof(log_id), "%s/%s", config.name, config.type);
+    p_redis_init(&redis_host, log_id, p_redis_thread_produce_common_plugin_handler);
+  }
+#endif
+
   /* plugin main loop */
   for(;;) {
     poll_again:
@@ -217,7 +238,6 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
     /* doing server tasks */
     if (poll_fd[1].revents & POLLIN) {
       struct pollfd pfd;
-      int ret;
 
       sd2 = accept(sd, &cAddr, &cLen);
       setblocking(sd2);
@@ -303,11 +323,16 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	    break;
           case 0: /* Child */
             close(sd);
+
 	    pm_setproctitle("%s [%s]", "IMT Plugin -- serving client", config.name);
+	    config.is_forked = TRUE;
+
             if (num > 0) process_query_data(sd2, srvbuf, num, &extras, datasize, TRUE);
 	    else Log(LOG_DEBUG, "DEBUG ( %s/%s ): %d incoming bytes. Errno: %d\n", config.name, config.type, num, errno);
+
             Log(LOG_DEBUG, "DEBUG ( %s/%s ): Closing connection with client ...\n", config.name, config.type);
             close(sd2);
+
             exit_gracefully(0);
           default: /* Parent */
             break;
@@ -349,8 +374,15 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
       reload_map = FALSE;
     }
 
+    if (reload_log) {
+      reload_logs();
+      reload_log = FALSE;
+    }
+
     if (poll_fd[0].revents & POLLIN) {
+#ifdef WITH_ZMQ
       read_data:
+#endif
       if (config.pipe_homegrown) {
         if (!pollagain) {
           seq++;
@@ -370,7 +402,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
         if (((struct ch_buf_hdr *)pipebuf)->seq != seq) {
           rg_err_count++;
           if (config.debug || (rg_err_count > MAX_RG_COUNT_ERR)) {
-	    Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%llu plugin_pipe_size=%llu).\n",
+	    Log(LOG_WARNING, "WARN ( %s/%s ): Missing data detected (plugin_buffer_size=%" PRIu64 " plugin_pipe_size=%" PRIu64 ").\n",
 		config.name, config.type, config.buffer_size, config.pipe_size);
 	    Log(LOG_WARNING, "WARN ( %s/%s ): Increase values or look for plugin_buffer_size, plugin_pipe_size in CONFIG-KEYS document.\n\n",
 		config.name, config.type);
@@ -398,7 +430,7 @@ void imt_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	data = (struct pkt_data *) (pipebuf+sizeof(struct ch_buf_hdr));
 
 	if (config.debug_internal_msg) 
-	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%llu seq=%u num_entries=%u\n",
+	  Log(LOG_DEBUG, "DEBUG ( %s/%s ): buffer received len=%" PRIu64 " seq=%u num_entries=%u\n",
 		config.name, config.type, ((struct ch_buf_hdr *)pipebuf)->len, seq,
 		((struct ch_buf_hdr *)pipebuf)->num);
 
