@@ -21,12 +21,9 @@
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#define __ISIS_LSP_C
-
 #include "pmacct.h"
 #include "isis.h"
 
-#include "linklist.h"
 #include "thread.h"
 #include "stream.h"
 #include "prefix.h"
@@ -98,27 +95,25 @@ lsp_clear_data (struct isis_lsp *lsp)
 	free(lsp->tlv_data.hostname);
     }
   if (lsp->tlv_data.is_neighs)
-    isis_list_delete (lsp->tlv_data.is_neighs);
+    pm_list_delete (lsp->tlv_data.is_neighs);
   if (lsp->tlv_data.te_is_neighs)
-    isis_list_delete (lsp->tlv_data.te_is_neighs);
+    pm_list_delete (lsp->tlv_data.te_is_neighs);
   if (lsp->tlv_data.area_addrs)
-    isis_list_delete (lsp->tlv_data.area_addrs);
+    pm_list_delete (lsp->tlv_data.area_addrs);
   if (lsp->tlv_data.es_neighs)
-    isis_list_delete (lsp->tlv_data.es_neighs);
+    pm_list_delete (lsp->tlv_data.es_neighs);
   if (lsp->tlv_data.ipv4_addrs)
-    isis_list_delete (lsp->tlv_data.ipv4_addrs);
+    pm_list_delete (lsp->tlv_data.ipv4_addrs);
   if (lsp->tlv_data.ipv4_int_reachs)
-    isis_list_delete (lsp->tlv_data.ipv4_int_reachs);
+    pm_list_delete (lsp->tlv_data.ipv4_int_reachs);
   if (lsp->tlv_data.ipv4_ext_reachs)
-    isis_list_delete (lsp->tlv_data.ipv4_ext_reachs);
+    pm_list_delete (lsp->tlv_data.ipv4_ext_reachs);
   if (lsp->tlv_data.te_ipv4_reachs)
-    isis_list_delete (lsp->tlv_data.te_ipv4_reachs);
-#ifdef ENABLE_IPV6
+    pm_list_delete (lsp->tlv_data.te_ipv4_reachs);
   if (lsp->tlv_data.ipv6_addrs)
-    isis_list_delete (lsp->tlv_data.ipv6_addrs);
+    pm_list_delete (lsp->tlv_data.ipv6_addrs);
   if (lsp->tlv_data.ipv6_reachs)
-    isis_list_delete (lsp->tlv_data.ipv6_reachs);
-#endif /* ENABLE_IPV6 */
+    pm_list_delete (lsp->tlv_data.ipv6_reachs);
 
   memset (&lsp->tlv_data, 0, sizeof (struct tlvs));
 
@@ -135,7 +130,7 @@ lsp_destroy (struct isis_lsp *lsp)
 
   if (LSP_FRAGMENT (lsp->lsp_header->lsp_id) == 0 && lsp->lspu.frags)
     {
-      isis_list_delete (lsp->lspu.frags);
+      pm_list_delete (lsp->lspu.frags);
     }
 
   if (lsp->pdu)
@@ -168,20 +163,20 @@ lsp_db_destroy (dict_t * lspdb)
  * Remove all the frags belonging to the given lsp
  */
 static void
-lsp_remove_frags (struct list *frags, dict_t * lspdb)
+lsp_remove_frags (struct pm_list *frags, dict_t * lspdb)
 {
   dnode_t *dnode;
-  struct listnode *lnode, *lnnode;
+  struct pm_listnode *lnode, *lnnode;
   struct isis_lsp *lsp;
 
-  for (ALL_LIST_ELEMENTS (frags, lnode, lnnode, lsp))
+  for (PM_ALL_LIST_ELEMENTS (frags, lnode, lnnode, lsp))
     {
       dnode = dict_lookup (lspdb, lsp->lsp_header->lsp_id);
       lsp_destroy (lsp);
       dnode_destroy (dict_delete (lspdb, dnode));
     }
 
-  isis_list_delete_all_node (frags);
+  pm_list_delete_all_node (frags);
 
   return;
 }
@@ -211,7 +206,7 @@ lsp_search_and_destroy (u_char * id, dict_t * lspdb)
 	   * else just remove this frag, from the zero lsps' frag list
 	   */
 	  if (lsp->lspu.zero_lsp && lsp->lspu.zero_lsp->lspu.frags)
-	    isis_listnode_delete (lsp->lspu.zero_lsp->lspu.frags, lsp);
+	    pm_listnode_delete (lsp->lspu.zero_lsp->lspu.frags, lsp);
 	}
       lsp_destroy (lsp);
       dnode_destroy (node);
@@ -303,14 +298,14 @@ static void
 lsp_seqnum_update (struct isis_lsp *lsp0)
 {
   struct isis_lsp *lsp;
-  struct listnode *node;
+  struct pm_listnode *node;
 
   lsp_inc_seqnum (lsp0, 0);
 
   if (!lsp0->lspu.frags)
     return;
 
-  for (ALL_LIST_ELEMENTS_RO (lsp0->lspu.frags, node, lsp))
+  for (PM_ALL_LIST_ELEMENTS_RO (lsp0->lspu.frags, node, lsp))
     lsp_inc_seqnum (lsp, 0);
 
   return;
@@ -371,15 +366,14 @@ lsp_update_data (struct isis_lsp *lsp, struct stream *stream,
   expected |= TLVFLAG_IPV4_ADDR;
   expected |= TLVFLAG_IPV4_INT_REACHABILITY;
   expected |= TLVFLAG_IPV4_EXT_REACHABILITY;
-#ifdef ENABLE_IPV6
   expected |= TLVFLAG_IPV6_ADDR;
   expected |= TLVFLAG_IPV6_REACHABILITY;
-#endif /* ENABLE_IPV6 */
 
   retval = parse_tlvs (area->area_tag, lsp->pdu->data +
 		       ISIS_FIXED_HDR_LEN + ISIS_LSP_HDR_LEN,
 		       ntohs (lsp->lsp_header->pdu_len) - ISIS_FIXED_HDR_LEN
 		       - ISIS_LSP_HDR_LEN, &expected, &found, &lsp->tlv_data);
+  (void)retval; //TODO treat error
 
   if (found & TLVFLAG_DYN_HOSTNAME)
     {
@@ -433,7 +427,7 @@ lsp_new_from_stream_ptr (struct stream *stream,
       /*
        * zero lsp -> create the list for fragments
        */
-      lsp->lspu.frags = isis_list_new ();
+      lsp->lspu.frags = pm_list_new ();
     }
   else
     {
@@ -441,7 +435,7 @@ lsp_new_from_stream_ptr (struct stream *stream,
        * a fragment -> set the backpointer and add this to zero lsps frag list
        */
       lsp->lspu.zero_lsp = lsp0;
-      isis_listnode_add (lsp0->lspu.frags, lsp);
+      pm_listnode_add (lsp0->lspu.frags, lsp);
     }
 
   return lsp;
@@ -467,7 +461,7 @@ lsp_new (u_char * lsp_id, u_int16_t rem_lifetime, u_int32_t seq_num,
   lsp->pdu = calloc(1, ISIS_FIXED_HDR_LEN + ISIS_LSP_HDR_LEN);
 #endif /* LSP_MEMORY_PREASSIGN */
   if (LSP_FRAGMENT (lsp_id) == 0)
-    lsp->lspu.frags = isis_list_new ();
+    lsp->lspu.frags = pm_list_new ();
   lsp->isis_header = (struct isis_fixed_hdr *) (STREAM_DATA (lsp->pdu));
   lsp->lsp_header = (struct isis_link_state_hdr *)
     (STREAM_DATA (lsp->pdu) + ISIS_FIXED_HDR_LEN);
@@ -509,7 +503,7 @@ lsp_insert (struct isis_lsp *lsp, dict_t * lspdb)
  */
 void
 lsp_build_isis_list_nonzero_ht (u_char * start_id, u_char * stop_id,
-			   struct list *list, dict_t * lspdb)
+			   struct pm_list *list, dict_t * lspdb)
 {
   dnode_t *first, *last, *curr;
 
@@ -522,14 +516,14 @@ lsp_build_isis_list_nonzero_ht (u_char * start_id, u_char * stop_id,
   curr = first;
 
   if (((struct isis_lsp *) (curr->dict_data))->lsp_header->rem_lifetime)
-    isis_listnode_add (list, first->dict_data);
+    pm_listnode_add (list, first->dict_data);
 
   while (curr)
     {
       curr = dict_next (lspdb, curr);
       if (curr &&
 	  ((struct isis_lsp *) (curr->dict_data))->lsp_header->rem_lifetime)
-	isis_listnode_add (list, curr->dict_data);
+	pm_listnode_add (list, curr->dict_data);
       if (curr == last)
 	break;
     }
@@ -542,7 +536,7 @@ lsp_build_isis_list_nonzero_ht (u_char * start_id, u_char * stop_id,
  */
 void
 lsp_build_list (u_char * start_id, u_char * stop_id,
-		struct list *list, dict_t * lspdb)
+		struct pm_list *list, dict_t * lspdb)
 {
   dnode_t *first, *last, *curr;
 
@@ -554,13 +548,13 @@ lsp_build_list (u_char * start_id, u_char * stop_id,
 
   curr = first;
 
-  isis_listnode_add (list, first->dict_data);
+  pm_listnode_add (list, first->dict_data);
 
   while (curr)
     {
       curr = dict_next (lspdb, curr);
       if (curr)
-	isis_listnode_add (list, curr->dict_data);
+	pm_listnode_add (list, curr->dict_data);
       if (curr == last)
 	break;
     }
@@ -572,7 +566,7 @@ lsp_build_list (u_char * start_id, u_char * stop_id,
  * Build a list of LSPs with SSN flag set for the given circuit
  */
 void
-lsp_build_isis_list_ssn (struct isis_circuit *circuit, struct list *list,
+lsp_build_isis_list_ssn (struct isis_circuit *circuit, struct pm_list *list,
 		    dict_t * lspdb)
 {
   dnode_t *dnode, *next;
@@ -584,7 +578,7 @@ lsp_build_isis_list_ssn (struct isis_circuit *circuit, struct list *list,
       next = dict_next (lspdb, dnode);
       lsp = dnode_get (dnode);
       if (ISIS_CHECK_FLAG (lsp->SSNflags, circuit))
-	isis_listnode_add (list, lsp);
+	pm_listnode_add (list, lsp);
       dnode = next;
     }
 
@@ -624,14 +618,14 @@ lsp_bits2string (u_char * lsp_bits)
 /* FIXME: It shouldn't be necessary to pass tlvsize here, TLVs can have
  * variable length (TE TLVs, sub TLVs). */
 static void
-lsp_tlv_fit (struct isis_lsp *lsp, struct list **from, struct list **to,
+lsp_tlv_fit (struct isis_lsp *lsp, struct pm_list **from, struct pm_list **to,
 	     int tlvsize, int frag_thold,
-	     int tlv_build_func (struct list *, struct stream *))
+	     int tlv_build_func (struct pm_list *, struct stream *))
 {
   int count, i;
 
   /* can we fit all ? */
-  if (!FRAG_NEEDED (lsp->pdu, frag_thold, listcount (*from) * tlvsize + 2))
+  if (!FRAG_NEEDED (lsp->pdu, frag_thold, pm_listcount (*from) * tlvsize + 2))
     {
       tlv_build_func (*from, lsp->pdu);
       *to = *from;
@@ -646,8 +640,8 @@ lsp_tlv_fit (struct isis_lsp *lsp, struct list **from, struct list **to,
 	count = count / tlvsize;
       for (i = 0; i < count; i++)
 	{
-	  isis_listnode_add (*to, listgetdata (listhead (*from)));
-	  isis_listnode_delete (*from, listgetdata (listhead (*from)));
+	  pm_listnode_add (*to, pm_listgetdata (pm_listhead (*from)));
+	  pm_listnode_delete (*from, pm_listgetdata (pm_listhead (*from)));
 	}
       tlv_build_func (*to, lsp->pdu);
     }
@@ -673,8 +667,10 @@ lsp_next_frag (u_char frag_num, struct isis_lsp *lsp0, struct isis_area *area,
       lsp_clear_data (lsp);
       if (lsp0->tlv_data.auth_info.type)
 	{
-	  memcpy (&lsp->tlv_data.auth_info, &lsp->tlv_data.auth_info,
+	  /* FIXME: this is broken. Detected by GCC 8.3
+           memcpy (&lsp->tlv_data.auth_info, &lsp->tlv_data.auth_info,
 		  sizeof (struct isis_passwd));
+          */
 	  tlv_add_authinfo (lsp->tlv_data.auth_info.type,
 			    lsp->tlv_data.auth_info.len,
 			    lsp->tlv_data.auth_info.passwd, lsp->pdu);
@@ -685,15 +681,17 @@ lsp_next_frag (u_char frag_num, struct isis_lsp *lsp0, struct isis_area *area,
 		 0, level);
   lsp->own_lsp = 1;
   lsp_insert (lsp, area->lspdb[level - 1]);
-  isis_listnode_add (lsp0->lspu.frags, lsp);
+  pm_listnode_add (lsp0->lspu.frags, lsp);
   lsp->lspu.zero_lsp = lsp0;
   /*
    * Copy the authinfo from zero LSP
    */
   if (lsp0->tlv_data.auth_info.type)
     {
+      /* FIXME: this is broken. Detected by GCC 8.3
       memcpy (&lsp->tlv_data.auth_info, &lsp->tlv_data.auth_info,
 	      sizeof (struct isis_passwd));
+      */
       tlv_add_authinfo (lsp->tlv_data.auth_info.type,
 			lsp->tlv_data.auth_info.len,
 			lsp->tlv_data.auth_info.passwd, lsp->pdu);
@@ -710,17 +708,15 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 {
   struct is_neigh *is_neigh;
   struct te_is_neigh *te_is_neigh;
-  struct listnode *node, *ipnode;
+  struct pm_listnode *node, *ipnode;
   int level = lsp->level;
   struct isis_circuit *circuit;
   struct prefix_ipv4 *ipv4;
   struct ipv4_reachability *ipreach;
   struct te_ipv4_reachability *te_ipreach;
   struct isis_adjacency *nei;
-#ifdef ENABLE_IPV6
   struct prefix_ipv6 *ipv6, *ip6prefix;
   struct ipv6_reachability *ip6reach;
-#endif /* ENABLE_IPV6 */
   struct tlvs tlv_data;
   struct isis_lsp *lsp0 = lsp;
   struct isis_passwd *passwd;
@@ -732,14 +728,10 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 
   /* Area addresses */
   if (lsp->tlv_data.area_addrs == NULL)
-    lsp->tlv_data.area_addrs = isis_list_new ();
-  isis_list_add_list (lsp->tlv_data.area_addrs, area->area_addrs);
+    lsp->tlv_data.area_addrs = pm_list_new ();
+  pm_list_add_list (lsp->tlv_data.area_addrs, area->area_addrs);
   /* Protocols Supported */
-  if (area->ip_circuits > 0
-#ifdef ENABLE_IPV6
-      || area->ipv6_circuits > 0
-#endif /* ENABLE_IPV6 */
-    )
+  if (area->ip_circuits > 0 || area->ipv6_circuits > 0)
     {
       lsp->tlv_data.nlpids = calloc(1, sizeof (struct nlpids));
       lsp->tlv_data.nlpids->count = 0;
@@ -748,14 +740,12 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	  lsp->tlv_data.nlpids->count++;
 	  lsp->tlv_data.nlpids->nlpids[0] = NLPID_IP;
 	}
-#ifdef ENABLE_IPV6
       if (area->ipv6_circuits > 0)
 	{
 	  lsp->tlv_data.nlpids->count++;
 	  lsp->tlv_data.nlpids->nlpids[lsp->tlv_data.nlpids->count - 1] =
 	    NLPID_IPV6;
 	}
-#endif /* ENABLE_IPV6 */
     }
   /* XXX: Dynamic Hostname */
 /*
@@ -793,7 +783,7 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
     tlv_add_nlpid (lsp->tlv_data.nlpids, lsp->pdu);
   if (lsp->tlv_data.hostname)
     tlv_add_dynamic_hostname (lsp->tlv_data.hostname, lsp->pdu);
-  if (lsp->tlv_data.area_addrs && listcount (lsp->tlv_data.area_addrs) > 0)
+  if (lsp->tlv_data.area_addrs && pm_listcount (lsp->tlv_data.area_addrs) > 0)
     tlv_add_area_addrs (lsp->tlv_data.area_addrs, lsp->pdu);
 
   /* IPv4 address and TE router ID TLVs. In case of the first one we don't
@@ -803,13 +793,13 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
     {
       if (lsp->tlv_data.ipv4_addrs == NULL)
 	{
-	  lsp->tlv_data.ipv4_addrs = isis_list_new ();
+	  lsp->tlv_data.ipv4_addrs = pm_list_new ();
 	  lsp->tlv_data.ipv4_addrs->del = free_tlv;
 	}
 
       routerid = calloc(1, sizeof (struct in_addr));
       routerid->s_addr = router_id_zebra.s_addr;
-      isis_listnode_add (lsp->tlv_data.ipv4_addrs, routerid);
+      pm_listnode_add (lsp->tlv_data.ipv4_addrs, routerid);
       tlv_add_in_addr (routerid, lsp->pdu, IPV4_ADDR);
 
       /* Exactly same data is put into TE router ID TLV, but only if new style
@@ -827,7 +817,7 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
   /*
    * Then build lists of tlvs related to circuits
    */
-  for (ALL_LIST_ELEMENTS_RO (area->circuit_list, node, circuit))
+  for (PM_ALL_LIST_ELEMENTS_RO (area->circuit_list, node, circuit))
     {
       if (circuit->state != C_STATE_UP)
 	continue;
@@ -842,17 +832,17 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	    {
 	      if (tlv_data.ipv4_int_reachs == NULL)
 		{
-		  tlv_data.ipv4_int_reachs = isis_list_new ();
+		  tlv_data.ipv4_int_reachs = pm_list_new ();
 		  tlv_data.ipv4_int_reachs->del = free_tlv;
 		}
-	      for (ALL_LIST_ELEMENTS_RO (circuit->ip_addrs, ipnode, ipv4))
+	      for (PM_ALL_LIST_ELEMENTS_RO (circuit->ip_addrs, ipnode, ipv4))
 		{
 		  ipreach = calloc(1, sizeof (struct ipv4_reachability));
 		  ipreach->metrics = circuit->metrics[level - 1];
 		  isis_masklen2ip (ipv4->prefixlen, &ipreach->mask);
 		  ipreach->prefix.s_addr = ((ipreach->mask.s_addr) &
 					    (ipv4->prefix.s_addr));
-		  isis_listnode_add (tlv_data.ipv4_int_reachs, ipreach);
+		  pm_listnode_add (tlv_data.ipv4_int_reachs, ipreach);
 		}
 	      tlv_data.ipv4_int_reachs->del = free_tlv;
 	    }
@@ -860,10 +850,10 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	    {
 	      if (tlv_data.te_ipv4_reachs == NULL)
 		{
-		  tlv_data.te_ipv4_reachs = isis_list_new ();
+		  tlv_data.te_ipv4_reachs = pm_list_new ();
 		  tlv_data.te_ipv4_reachs->del = free_tlv;
 		}
-	      for (ALL_LIST_ELEMENTS_RO (circuit->ip_addrs, ipnode, ipv4))
+	      for (PM_ALL_LIST_ELEMENTS_RO (circuit->ip_addrs, ipnode, ipv4))
 		{
 		  /* FIXME All this assumes that we have no sub TLVs. */
 		  te_ipreach = calloc(1, sizeof (struct te_ipv4_reachability) +
@@ -877,11 +867,10 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 		  te_ipreach->control = (ipv4->prefixlen & 0x3F);
 		  memcpy (&te_ipreach->prefix_start, &ipv4->prefix.s_addr,
 			  (ipv4->prefixlen + 7)/8);
-		  isis_listnode_add (tlv_data.te_ipv4_reachs, te_ipreach);
+		  pm_listnode_add (tlv_data.te_ipv4_reachs, te_ipreach);
 		}
 	    }
 	}
-#ifdef ENABLE_IPV6
       /*
        * Add IPv6 reachability of this circuit
        */
@@ -891,10 +880,10 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 
 	  if (tlv_data.ipv6_reachs == NULL)
 	    {
-	      tlv_data.ipv6_reachs = isis_list_new ();
+	      tlv_data.ipv6_reachs = pm_list_new ();
 	      tlv_data.ipv6_reachs->del = free_tlv;
 	    }
-          for (ALL_LIST_ELEMENTS_RO (circuit->ipv6_non_link, ipnode, ipv6))
+          for (PM_ALL_LIST_ELEMENTS_RO (circuit->ipv6_non_link, ipnode, ipv6))
 	    {
 	      ip6reach = calloc(1, sizeof (struct ipv6_reachability));
 
@@ -910,10 +899,9 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	      isis_apply_mask_ipv6 (ip6prefix);
 	      memcpy (ip6reach->prefix, ip6prefix->prefix.s6_addr,
 		      sizeof (ip6reach->prefix));
-	      isis_listnode_add (tlv_data.ipv6_reachs, ip6reach);
+	      pm_listnode_add (tlv_data.ipv6_reachs, ip6reach);
 	    }
 	}
-#endif /* ENABLE_IPV6 */
 
       switch (circuit->circ_type)
 	{
@@ -924,7 +912,7 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 		{
 		  if (tlv_data.is_neighs == NULL)
 		    {
-		      tlv_data.is_neighs = isis_list_new ();
+		      tlv_data.is_neighs = pm_list_new ();
 		      tlv_data.is_neighs->del = free_tlv;
 		    }
 		  is_neigh = calloc(1, sizeof (struct is_neigh));
@@ -935,7 +923,7 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 		    memcpy (is_neigh->neigh_id,
 			    circuit->u.bc.l2_desig_is, ISIS_SYS_ID_LEN + 1);
 		  is_neigh->metrics = circuit->metrics[level - 1];
-		  isis_listnode_add (tlv_data.is_neighs, is_neigh);
+		  pm_listnode_add (tlv_data.is_neighs, is_neigh);
 		  tlv_data.is_neighs->del = free_tlv;
 		}
 	      if (area->newmetric)
@@ -944,7 +932,7 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 
 		  if (tlv_data.te_is_neighs == NULL)
 		    {
-		      tlv_data.te_is_neighs = isis_list_new ();
+		      tlv_data.te_is_neighs = pm_list_new ();
 		      tlv_data.te_is_neighs->del = free_tlv;
 		    }
 		  te_is_neigh = calloc(1, sizeof (struct te_is_neigh));
@@ -962,7 +950,7 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 		    metric = ((htonl(*circuit->te_metric) >> 8) & 0xffffff);
 
 		  memcpy (te_is_neigh->te_metric, &metric, 3);
-		  isis_listnode_add (tlv_data.te_is_neighs, te_is_neigh);
+		  pm_listnode_add (tlv_data.te_is_neighs, te_is_neigh);
 		}
 	    }
 	  break;
@@ -974,13 +962,13 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 		{
 		  if (tlv_data.is_neighs == NULL)
 		    {
-		      tlv_data.is_neighs = isis_list_new ();
+		      tlv_data.is_neighs = pm_list_new ();
 		      tlv_data.is_neighs->del = free_tlv;
 		    }
 		  is_neigh = calloc(1, sizeof (struct is_neigh));
 		  memcpy (is_neigh->neigh_id, nei->sysid, ISIS_SYS_ID_LEN);
 		  is_neigh->metrics = circuit->metrics[level - 1];
-		  isis_listnode_add (tlv_data.is_neighs, is_neigh);
+		  pm_listnode_add (tlv_data.is_neighs, is_neigh);
 		}
 	      if (area->newmetric)
 		{
@@ -988,14 +976,14 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 
 		  if (tlv_data.te_is_neighs == NULL)
 		    {
-		      tlv_data.te_is_neighs = isis_list_new ();
+		      tlv_data.te_is_neighs = pm_list_new ();
 		      tlv_data.te_is_neighs->del = free_tlv;
 		    }
 		  te_is_neigh = calloc(1, sizeof (struct te_is_neigh));
 		  memcpy (te_is_neigh->neigh_id, nei->sysid, ISIS_SYS_ID_LEN);
 		  metric = ((htonl(*circuit->te_metric) >> 8) & 0xffffff);
 		  memcpy (te_is_neigh->te_metric, &metric, 3);
-		  isis_listnode_add (tlv_data.te_is_neighs, te_is_neigh);
+		  pm_listnode_add (tlv_data.te_is_neighs, te_is_neigh);
 		}
 	    }
 	  break;
@@ -1013,69 +1001,67 @@ lsp_build_nonpseudo (struct isis_lsp *lsp, struct isis_area *area)
 	}
     }
 
-  while (tlv_data.ipv4_int_reachs && listcount (tlv_data.ipv4_int_reachs))
+  while (tlv_data.ipv4_int_reachs && pm_listcount (tlv_data.ipv4_int_reachs))
     {
       if (lsp->tlv_data.ipv4_int_reachs == NULL)
-	lsp->tlv_data.ipv4_int_reachs = isis_list_new ();
+	lsp->tlv_data.ipv4_int_reachs = pm_list_new ();
       lsp_tlv_fit (lsp, &tlv_data.ipv4_int_reachs,
 		   &lsp->tlv_data.ipv4_int_reachs,
 		   IPV4_REACH_LEN, area->lsp_frag_threshold,
 		   tlv_add_ipv4_reachs);
-      if (tlv_data.ipv4_int_reachs && listcount (tlv_data.ipv4_int_reachs))
+      if (tlv_data.ipv4_int_reachs && pm_listcount (tlv_data.ipv4_int_reachs))
 	lsp = lsp_next_frag (LSP_FRAGMENT (lsp->lsp_header->lsp_id) + 1,
 			     lsp0, area, level);
     }
   /* FIXME: We pass maximum te_ipv4_reachability length to the lsp_tlv_fit()
    * for now. lsp_tlv_fit() needs to be fixed to deal with variable length
    * TLVs (sub TLVs!). */
-  while (tlv_data.te_ipv4_reachs && listcount (tlv_data.te_ipv4_reachs))
+  while (tlv_data.te_ipv4_reachs && pm_listcount (tlv_data.te_ipv4_reachs))
     {
       if (lsp->tlv_data.te_ipv4_reachs == NULL)
-	lsp->tlv_data.te_ipv4_reachs = isis_list_new ();
+	lsp->tlv_data.te_ipv4_reachs = pm_list_new ();
       lsp_tlv_fit (lsp, &tlv_data.te_ipv4_reachs,
 		   &lsp->tlv_data.te_ipv4_reachs,
 		   9, area->lsp_frag_threshold, tlv_add_te_ipv4_reachs);
-      if (tlv_data.te_ipv4_reachs && listcount (tlv_data.te_ipv4_reachs))
+      if (tlv_data.te_ipv4_reachs && pm_listcount (tlv_data.te_ipv4_reachs))
 	lsp = lsp_next_frag (LSP_FRAGMENT (lsp->lsp_header->lsp_id) + 1,
 			     lsp0, area, level);
     }
 
-#ifdef  ENABLE_IPV6
-  while (tlv_data.ipv6_reachs && listcount (tlv_data.ipv6_reachs))
+  while (tlv_data.ipv6_reachs && pm_listcount (tlv_data.ipv6_reachs))
     {
       if (lsp->tlv_data.ipv6_reachs == NULL)
-	lsp->tlv_data.ipv6_reachs = isis_list_new ();
+	lsp->tlv_data.ipv6_reachs = pm_list_new ();
       lsp_tlv_fit (lsp, &tlv_data.ipv6_reachs,
 		   &lsp->tlv_data.ipv6_reachs,
 		   IPV6_REACH_LEN, area->lsp_frag_threshold,
 		   tlv_add_ipv6_reachs);
-      if (tlv_data.ipv6_reachs && listcount (tlv_data.ipv6_reachs))
+      if (tlv_data.ipv6_reachs && pm_listcount (tlv_data.ipv6_reachs))
 	lsp = lsp_next_frag (LSP_FRAGMENT (lsp->lsp_header->lsp_id) + 1,
 			     lsp0, area, level);
     }
-#endif /* ENABLE_IPV6 */
 
-  while (tlv_data.is_neighs && listcount (tlv_data.is_neighs))
+  while (tlv_data.is_neighs && pm_listcount (tlv_data.is_neighs))
     {
       if (lsp->tlv_data.is_neighs == NULL)
-	lsp->tlv_data.is_neighs = isis_list_new ();
+	lsp->tlv_data.is_neighs = pm_list_new ();
       lsp_tlv_fit (lsp, &tlv_data.is_neighs,
 		   &lsp->tlv_data.is_neighs,
 		   IS_NEIGHBOURS_LEN, area->lsp_frag_threshold,
 		   tlv_add_is_neighs);
-      if (tlv_data.is_neighs && listcount (tlv_data.is_neighs))
+      if (tlv_data.is_neighs && pm_listcount (tlv_data.is_neighs))
 	lsp = lsp_next_frag (LSP_FRAGMENT (lsp->lsp_header->lsp_id) + 1,
 			     lsp0, area, level);
     }
 
-  while (tlv_data.te_is_neighs && listcount (tlv_data.te_is_neighs))
+  while (tlv_data.te_is_neighs && pm_listcount (tlv_data.te_is_neighs))
     {
       if (lsp->tlv_data.te_is_neighs == NULL)
-	lsp->tlv_data.te_is_neighs = isis_list_new ();
+	lsp->tlv_data.te_is_neighs = pm_list_new ();
       lsp_tlv_fit (lsp, &tlv_data.te_is_neighs, &lsp->tlv_data.te_is_neighs,
 		   IS_NEIGHBOURS_LEN, area->lsp_frag_threshold,
 		   tlv_add_te_is_neighs);
-      if (tlv_data.te_is_neighs && listcount (tlv_data.te_is_neighs))
+      if (tlv_data.te_is_neighs && pm_listcount (tlv_data.te_is_neighs))
 	lsp = lsp_next_frag (LSP_FRAGMENT (lsp->lsp_header->lsp_id) + 1,
 			     lsp0, area, level);
     }
@@ -1158,7 +1144,7 @@ lsp_non_pseudo_regenerate (struct isis_area *area, int level)
 {
   dict_t *lspdb = area->lspdb[level - 1];
   struct isis_lsp *lsp, *frag;
-  struct listnode *node;
+  struct pm_listnode *node;
   u_char lspid[ISIS_SYS_ID_LEN + 2];
 
   memset (lspid, 0, ISIS_SYS_ID_LEN + 2);
@@ -1194,7 +1180,7 @@ lsp_non_pseudo_regenerate (struct isis_area *area, int level)
   lsp->last_generated = time (NULL);
   area->lsp_regenerate_pending[level - 1] = 0;
   ISIS_FLAGS_SET_ALL (lsp->SRMflags);
-  for (ALL_LIST_ELEMENTS_RO (lsp->lspu.frags, node, frag))
+  for (PM_ALL_LIST_ELEMENTS_RO (lsp->lspu.frags, node, frag))
     {
       frag->lsp_header->rem_lifetime = htons (isis_jitter
 					      (area->
@@ -1205,10 +1191,9 @@ lsp_non_pseudo_regenerate (struct isis_area *area, int level)
 
   if (area->ip_circuits)
     isis_spf_schedule (area, level);
-#ifdef ENABLE_IPV6
   if (area->ipv6_circuits)
     isis_spf_schedule6 (area, level);
-#endif
+
   return ISIS_OK;
 }
 
@@ -1359,8 +1344,8 @@ lsp_build_pseudo (struct isis_lsp *lsp, struct isis_circuit *circuit,
   struct is_neigh *is_neigh;
   struct te_is_neigh *te_is_neigh;
   struct es_neigh *es_neigh;
-  struct list *adj_list;
-  struct listnode *node;
+  struct pm_list *adj_list;
+  struct pm_listnode *node;
   struct isis_passwd *passwd;
 
   assert (circuit);
@@ -1382,31 +1367,31 @@ lsp_build_pseudo (struct isis_lsp *lsp, struct isis_circuit *circuit,
     {
       if (lsp->tlv_data.is_neighs == NULL)
 	{
-	  lsp->tlv_data.is_neighs = isis_list_new ();
+	  lsp->tlv_data.is_neighs = pm_list_new ();
 	  lsp->tlv_data.is_neighs->del = free_tlv;
 	}
       is_neigh = calloc(1, sizeof (struct is_neigh));
 
       memcpy (&is_neigh->neigh_id, isis->sysid, ISIS_SYS_ID_LEN);
-      isis_listnode_add (lsp->tlv_data.is_neighs, is_neigh);
+      pm_listnode_add (lsp->tlv_data.is_neighs, is_neigh);
     }
   if (circuit->area->newmetric)
     {
       if (lsp->tlv_data.te_is_neighs == NULL)
 	{
-	  lsp->tlv_data.te_is_neighs = isis_list_new ();
+	  lsp->tlv_data.te_is_neighs = pm_list_new ();
 	  lsp->tlv_data.te_is_neighs->del = free_tlv;
 	}
       te_is_neigh = calloc(1, sizeof (struct te_is_neigh));
 
       memcpy (&te_is_neigh->neigh_id, isis->sysid, ISIS_SYS_ID_LEN);
-      isis_listnode_add (lsp->tlv_data.te_is_neighs, te_is_neigh);
+      pm_listnode_add (lsp->tlv_data.te_is_neighs, te_is_neigh);
     }
 
-  adj_list = isis_list_new ();
+  adj_list = pm_list_new ();
   isis_adj_build_up_list (circuit->u.bc.adjdb[level - 1], adj_list);
 
-  for (ALL_LIST_ELEMENTS_RO (adj_list, node, adj))
+  for (PM_ALL_LIST_ELEMENTS_RO (adj_list, node, adj))
     {
       if (adj->circuit_t & level)
 	{
@@ -1421,13 +1406,13 @@ lsp_build_pseudo (struct isis_lsp *lsp, struct isis_circuit *circuit,
 		  is_neigh = calloc(1, sizeof (struct is_neigh));
 
 		  memcpy (&is_neigh->neigh_id, adj->sysid, ISIS_SYS_ID_LEN);
-		  isis_listnode_add (lsp->tlv_data.is_neighs, is_neigh);
+		  pm_listnode_add (lsp->tlv_data.is_neighs, is_neigh);
 		}
 	      if (circuit->area->newmetric)
 		{
 		  te_is_neigh = calloc(1, sizeof (struct te_is_neigh));
 		  memcpy (&te_is_neigh->neigh_id, adj->sysid, ISIS_SYS_ID_LEN);
-		  isis_listnode_add (lsp->tlv_data.te_is_neighs, te_is_neigh);
+		  pm_listnode_add (lsp->tlv_data.te_is_neighs, te_is_neigh);
 		}
 	    }
 	  else if (level == 1 && adj->sys_type == ISIS_SYSTYPE_ES)
@@ -1436,13 +1421,13 @@ lsp_build_pseudo (struct isis_lsp *lsp, struct isis_circuit *circuit,
 	      /* FIXME: the tlv-format is hard to use here */
 	      if (lsp->tlv_data.es_neighs == NULL)
 		{
-		  lsp->tlv_data.es_neighs = isis_list_new ();
+		  lsp->tlv_data.es_neighs = pm_list_new ();
 		  lsp->tlv_data.es_neighs->del = free_tlv;
 		}
 	      es_neigh = calloc(1, sizeof (struct es_neigh));
 	      
 	      memcpy (&es_neigh->first_es_neigh, adj->sysid, ISIS_SYS_ID_LEN);
-	      isis_listnode_add (lsp->tlv_data.es_neighs, es_neigh);
+	      pm_listnode_add (lsp->tlv_data.es_neighs, es_neigh);
 	    }
 	}
     }
@@ -1462,20 +1447,20 @@ lsp_build_pseudo (struct isis_lsp *lsp, struct isis_circuit *circuit,
       tlv_add_authinfo (passwd->type, passwd->len, passwd->passwd, lsp->pdu);
     }
 
-  if (lsp->tlv_data.is_neighs && listcount (lsp->tlv_data.is_neighs) > 0)
+  if (lsp->tlv_data.is_neighs && pm_listcount (lsp->tlv_data.is_neighs) > 0)
     tlv_add_is_neighs (lsp->tlv_data.is_neighs, lsp->pdu);
 
-  if (lsp->tlv_data.te_is_neighs && listcount (lsp->tlv_data.te_is_neighs) > 0)
+  if (lsp->tlv_data.te_is_neighs && pm_listcount (lsp->tlv_data.te_is_neighs) > 0)
     tlv_add_te_is_neighs (lsp->tlv_data.te_is_neighs, lsp->pdu);
 
-  if (lsp->tlv_data.es_neighs && listcount (lsp->tlv_data.es_neighs) > 0)
+  if (lsp->tlv_data.es_neighs && pm_listcount (lsp->tlv_data.es_neighs) > 0)
     tlv_add_is_neighs (lsp->tlv_data.es_neighs, lsp->pdu);
 
   lsp->lsp_header->pdu_len = htons (stream_get_endp (lsp->pdu));
   fletcher_checksum (STREAM_DATA (lsp->pdu) + 12,
 		   ntohs (lsp->lsp_header->pdu_len) - 12, 12);
 
-  isis_list_delete (adj_list);
+  pm_list_delete (adj_list);
 
   return;
 }
