@@ -521,7 +521,7 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
 
 	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_INIT_SID], &p_avro_value, &p_avro_local_buf,
 					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
-	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed for %s: %s\n", config.name, config.type, "acct_init", kafkap_kafka_host.errstr);
 	    exit_gracefully(1);
 	  }
 	  else {
@@ -639,7 +639,7 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
 
 	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_DATA_SID], &p_avro_value, &p_avro_local_buf,
 					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
-	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed for %s: %s\n", config.name, config.type, "acct_data", kafkap_kafka_host.errstr);
 	    exit_gracefully(1);
 	  }
 	  else {
@@ -847,7 +847,7 @@ void kafka_cache_purge(struct chained_cache *queue[], int index, int safe_action
 
 	  if (serdes_schema_serialize_avro(kafkap_kafka_host.sd_schema[AVRO_ACCT_CLOSE_SID], &p_avro_value, &p_avro_local_buf,
 					   &p_avro_len, kafkap_kafka_host.errstr, sizeof(kafkap_kafka_host.errstr))) {
-	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed: %s\n", config.name, config.type, kafkap_kafka_host.errstr);
+	    Log(LOG_ERR, "ERROR ( %s/%s ): kafka_cache_purge(): serdes_schema_serialize_avro() failed for %s: %s\n", config.name, config.type, "acct_close", kafkap_kafka_host.errstr);
 	    exit_gracefully(1);
 	  }
 	  else {
