@@ -170,7 +170,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 
     /* lazy sql refresh handling */
     if (idata.now > refresh_deadline) {
-      if (qq_ptr) sql_cache_flush(sql_queries_queue, qq_ptr, &idata, FALSE);
+      if (sql_qq_ptr) sql_cache_flush(sql_queries_queue, sql_qq_ptr, &idata, FALSE);
       sql_cache_handle_flush_event(&idata, &refresh_deadline, &pt);
     }
     else {
@@ -272,7 +272,7 @@ void mysql_plugin(int pipe_fd, struct configuration *cfgptr, void *ptr)
 	}
 
         prim_ptrs.data = data;
-	(*insert_func)(&prim_ptrs, &idata);
+	(*sql_insert_func)(&prim_ptrs, &idata);
 	
 	((struct ch_buf_hdr *)pipebuf)->num--;
         if (((struct ch_buf_hdr *)pipebuf)->num) {
@@ -469,7 +469,7 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
 
   /* re-using pending queries queue stuff from parent and saving clauses */
   memcpy(sql_pending_queries_queue, queue, index*sizeof(struct db_cache *));
-  pqq_ptr = index;
+  sql_pqq_ptr = index;
 
   strlcpy(orig_insert_clause, insert_clause, LONGSRVBUFLEN);
   strlcpy(orig_update_clause, update_clause, LONGSRVBUFLEN);
@@ -477,9 +477,9 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
 
   start:
   memset(&idata->mv, 0, sizeof(struct multi_values));
-  memcpy(queue, sql_pending_queries_queue, pqq_ptr*sizeof(struct db_cache *));
-  memset(sql_pending_queries_queue, 0, pqq_ptr*sizeof(struct db_cache *));
-  index = pqq_ptr; pqq_ptr = 0;
+  memcpy(queue, sql_pending_queries_queue, sql_pqq_ptr*sizeof(struct db_cache *));
+  memset(sql_pending_queries_queue, 0, sql_pqq_ptr*sizeof(struct db_cache *));
+  index = sql_pqq_ptr; sql_pqq_ptr = 0;
 
   /* We check for variable substitution in SQL table */ 
   if (idata->dyn_table) {
@@ -526,9 +526,9 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
       pm_strftime_same(tmptable, LONGSRVBUFLEN, tmpbuf, &stamp, config.timestamps_utc);
 
       if (strncmp(idata->dyn_table_name, tmptable, SRVBUFLEN)) {
-	sql_pending_queries_queue[pqq_ptr] = queue[idata->current_queue_elem];
+	sql_pending_queries_queue[sql_pqq_ptr] = queue[idata->current_queue_elem];
 
-	pqq_ptr++;
+	sql_pqq_ptr++;
         go_to_pending = TRUE;
       }
     }
@@ -553,7 +553,7 @@ void MY_cache_purge(struct db_cache *queue[], int index, struct insert_data *ida
   if (b.fail) Log(LOG_ALERT, "ALERT ( %s/%s ): recovery for MySQL daemon failed.\n", config.name, config.type);
 
   /* If we have pending queries then start again */
-  if (pqq_ptr) goto start;
+  if (sql_pqq_ptr) goto start;
   
   idata->elap_time = time(NULL)-start; 
   Log(LOG_INFO, "INFO ( %s/%s ): *** Purging cache - END (PID: %u, QN: %u/%u, ET: %lu) ***\n", 
@@ -686,6 +686,8 @@ void MY_Unlock(struct BE_descs *bed)
 void MY_DB_Connect(struct DBdesc *db, char *host)
 {
   bool reconnect = TRUE;
+  void *mysql_ret = NULL;
+  char *usp = NULL;
 
   if (!db->fail) {
     mysql_init(db->desc);
@@ -693,12 +695,23 @@ void MY_DB_Connect(struct DBdesc *db, char *host)
     mysql_options(db->desc, MYSQL_OPT_RECONNECT, &reconnect);
     if (config.sql_conn_ca_file) mysql_ssl_set(db->desc, NULL, NULL, config.sql_conn_ca_file, NULL, NULL);
     
-    if (!mysql_real_connect(db->desc, host, config.sql_user, config.sql_passwd, config.sql_db, config.sql_port, NULL, 0)) {
+    usp = strstr(host, "unix:");
+    if (usp && usp == host) {
+      usp += 5; /* go right past the 'unix:' string */
+      mysql_ret = mysql_real_connect(db->desc, NULL, config.sql_user, config.sql_passwd, config.sql_db, FALSE, usp, 0);
+    }
+    else {
+      mysql_ret = mysql_real_connect(db->desc, host, config.sql_user, config.sql_passwd, config.sql_db, config.sql_port, NULL, 0);
+    }
+
+    if (!mysql_ret) {
       sql_db_fail(db);
       MY_get_errmsg(db);
       sql_db_errmsg(db);
     }
-    else sql_db_ok(db);
+    else {
+      sql_db_ok(db);
+    }
   }
 }
 
