@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2018 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2020 by Paolo Lucente
 */
 
 /*
@@ -24,14 +24,14 @@
     ndpi.c ndpiReader.c | nDPI | Copyright (C) 2011-17 - ntop.org
 */
 
-#define __NDPI_C
-
-#ifdef WITH_NDPI
 #include "../pmacct.h"
 #include "../ip_flow.h"
 #include "../classifier.h"
 #include "ndpi.h"
 
+/* Global variables */
+struct pm_ndpi_workflow *pm_ndpi_wfl;
+
 void pm_ndpi_free_flow_info_half(struct pm_ndpi_flow_info *flow)
 {
   if (flow) {
@@ -372,8 +372,9 @@ struct ndpi_proto pm_ndpi_packet_processing(struct pm_ndpi_workflow *workflow,
   }
 
   if (flow->detection_completed || flow->tcp_finished) {
-    if (flow->detected_protocol.app_protocol == NDPI_PROTOCOL_UNKNOWN)
-      flow->detected_protocol = ndpi_detection_giveup(workflow->ndpi_struct, flow->ndpi_flow);
+    if (flow->detected_protocol.app_protocol == NDPI_PROTOCOL_UNKNOWN) {
+      flow->detected_protocol = ndpi_detection_giveup(workflow->ndpi_struct, flow->ndpi_flow, 1, &workflow->prefs.protocol_guess);
+    }
 
     if (workflow->prefs.protocol_guess) {
       if (flow->detected_protocol.app_protocol == NDPI_PROTOCOL_UNKNOWN && !flow->guess_completed) {
@@ -440,6 +441,7 @@ u_int16_t pm_ndpi_node_guess_undetected_protocol(struct pm_ndpi_workflow *workfl
   if (!flow || !workflow) return 0;
 
   flow->detected_protocol = ndpi_guess_undetected_protocol(workflow->ndpi_struct,
+							   flow->ndpi_flow,
                                                            flow->protocol,
                                                            ntohl(flow->lower_ip),
                                                            ntohs(flow->lower_port),
@@ -461,7 +463,7 @@ int pm_ndpi_node_idle_scan_walker(const void *node, const pm_VISIT which, const
 
   if (workflow->num_idle_flows == workflow->prefs.idle_scan_budget) return FALSE;
 
-  if ((which == ndpi_preorder) || (which == ndpi_leaf)) { /* Avoid walking the same node multiple times */
+  if ((which == (pm_VISIT)ndpi_preorder) || (which == (pm_VISIT)ndpi_leaf)) { /* Avoid walking the same node multiple times */
     /* expire Idle and TCP finished flows */
     if ((flow->last_seen + workflow->prefs.idle_max_time < workflow->last_time) ||
 	(flow->tcp_finished == TRUE)) {
@@ -496,4 +498,3 @@ void pm_ndpi_idle_flows_cleanup(struct pm_ndpi_workflow *workflow)
     workflow->last_idle_scan_time = workflow->last_time;
   }
 }
-#endif
