@@ -198,6 +198,9 @@ struct data_hdr_v9 {
 #define NF9_FLOW_PACKETS		86 
 
 #define NF9_FORWARDING_STATUS           89
+#define NF9_MPLS_VPN_RD			90
+/* ... */
+#define NF9_LAYER2_PKT_SECTION_DATA	104
 /* ... */
 #define NF9_PEER_DST_AS			128
 #define NF9_PEER_SRC_AS			129
@@ -314,7 +317,7 @@ struct data_hdr_v9 {
 #define NF9_CUST_LABEL			3
 /* CUSTOM TYPES END HERE */
 
-#define MAX_TPL_DESC_LIST 89
+#define MAX_TPL_DESC_LIST 90
 static char __attribute__((unused)) *tpl_desc_list[] = {
   "",
   "in bytes",
@@ -384,6 +387,7 @@ static char __attribute__((unused)) *tpl_desc_list[] = {
   "", "", "", "",
   "", "", "",
   "forwarding status",
+  "mpls vpn rd"
 };
 
 #define MAX_OPT_TPL_DESC_LIST 100
