@@ -32,9 +32,6 @@
 #include "classifier.h"
 #include "net_aggr.h"
 
-/* global var */
-struct channels_list_entry channels_list[MAX_N_PLUGINS]; /* communication channels: core <-> plugins */
-
 /* Functions */
 void usage_daemon(char *prog_name)
 {
@@ -96,6 +93,7 @@ int main(int argc,char **argv, char **envp)
 
   log_notifications_init(&log_notifications);
   config.acct_type = ACCT_PMTELE;
+  config.progname = pmtele_globstr;
 
   find_id_func = NULL;
   plugins_list = NULL;
@@ -191,6 +189,7 @@ int main(int argc,char **argv, char **envp)
   list = plugins_list;
   while (list) {
     list->cfg.acct_type = ACCT_PMTELE;
+    list->cfg.progname = pmtele_globstr;
     set_default_preferences(&list->cfg);
     if (!strcmp(list->type.string, "core")) {
       memcpy(&config, &list->cfg, sizeof(struct configuration));
@@ -203,6 +202,7 @@ int main(int argc,char **argv, char **envp)
   if (config.files_umask) umask(config.files_umask);
 
   initsetproctitle(argc, argv, envp);
+
   if (config.syslog) {
     logf = parse_log_facility(config.syslog);
     if (logf == ERR) {
